@@ -1,7 +1,8 @@
 import warnings as _warnings
 from typing import Optional as _Optional
 
-from ground.base import get_context as _get_context
+from ground.base import (Context as _Context,
+                         get_context as _get_context)
 from ground.hints import (Box as _Box,
                           Contour as _Contour,
                           Coordinate as _Coordinate,
@@ -40,8 +41,9 @@ from .hints import (Mix as _Mix,
 
 
 def points(x_coordinates: _Strategy[_Coordinate],
-           y_coordinates: _Optional[_Strategy[_Coordinate]] = None
-           ) -> _Strategy[_Point]:
+           y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
+           *,
+           context: _Optional[_Context] = None) -> _Strategy[_Point]:
     """
     Returns a strategy for points.
 
@@ -49,6 +51,7 @@ def points(x_coordinates: _Strategy[_Coordinate],
     :param y_coordinates:
         strategy for points' y-coordinates,
         ``None`` for reusing x-coordinates strategy.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -97,14 +100,15 @@ def points(x_coordinates: _Strategy[_Coordinate],
     True
     """
     return _points(x_coordinates, y_coordinates,
-                   context=_get_context())
+                   context=_get_context() if context is None else context)
 
 
 def multipoints(x_coordinates: _Strategy[_Coordinate],
                 y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
                 *,
                 min_size: int = 0,
-                max_size: _Optional[int] = None) -> _Strategy[_Multipoint]:
+                max_size: _Optional[int] = None,
+                context: _Optional[_Context] = None) -> _Strategy[_Multipoint]:
     """
     Returns a strategy for multipoints.
 
@@ -114,6 +118,7 @@ def multipoints(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for multipoint size.
     :param max_size: upper bound for multipoint size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -176,7 +181,8 @@ def multipoints(x_coordinates: _Strategy[_Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, 0)
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return _multipoints(x_coordinates, y_coordinates,
                         min_size=max(min_size, 0),
                         max_size=max_size,
@@ -184,8 +190,9 @@ def multipoints(x_coordinates: _Strategy[_Coordinate],
 
 
 def segments(x_coordinates: _Strategy[_Coordinate],
-             y_coordinates: _Optional[_Strategy[_Coordinate]] = None
-             ) -> _Strategy[_Segment]:
+             y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
+             *,
+             context: _Optional[_Context] = None) -> _Strategy[_Segment]:
     """
     Returns a strategy for segments.
 
@@ -193,6 +200,7 @@ def segments(x_coordinates: _Strategy[_Coordinate],
     :param y_coordinates:
         strategy for endpoints' y-coordinates,
         ``None`` for reusing x-coordinates strategy.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -249,14 +257,16 @@ def segments(x_coordinates: _Strategy[_Coordinate],
     True
     """
     return _segments(x_coordinates, y_coordinates,
-                     context=_get_context())
+                     context=_get_context() if context is None else context)
 
 
 def multisegments(x_coordinates: _Strategy[_Coordinate],
                   y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
                   *,
                   min_size: int = 0,
-                  max_size: _Optional[int] = None) -> _Strategy[_Multisegment]:
+                  max_size: _Optional[int] = None,
+                  context: _Optional[_Context] = None
+                  ) -> _Strategy[_Multisegment]:
     """
     Returns a strategy for multisegments.
 
@@ -266,6 +276,7 @@ def multisegments(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for multisegment size.
     :param max_size: upper bound for multisegment size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -339,14 +350,17 @@ def multisegments(x_coordinates: _Strategy[_Coordinate],
     return _multisegments(x_coordinates, y_coordinates,
                           min_size=max(min_size, 0),
                           max_size=max_size,
-                          context=_get_context())
+                          context=(_get_context()
+                                   if context is None
+                                   else context))
 
 
 def contours(x_coordinates: _Strategy[_Coordinate],
              y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
              *,
              min_size: int = _MIN_CONTOUR_SIZE,
-             max_size: _Optional[int] = None) -> _Strategy[_Contour]:
+             max_size: _Optional[int] = None,
+             context: _Optional[_Context] = None) -> _Strategy[_Contour]:
     """
     Returns a strategy for contours.
 
@@ -356,6 +370,7 @@ def contours(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for contour size.
     :param max_size: upper bound for contour size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -418,7 +433,8 @@ def contours(x_coordinates: _Strategy[_Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, _MIN_CONTOUR_SIZE)
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_vertices_sequences(x_coordinates, y_coordinates,
                                 min_size=max(min_size, _MIN_CONTOUR_SIZE),
                                 max_size=max_size,
@@ -430,7 +446,9 @@ def convex_contours(x_coordinates: _Strategy[_Coordinate],
                     y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
                     *,
                     min_size: int = _MinContourSize.CONVEX,
-                    max_size: _Optional[int] = None) -> _Strategy[_Contour]:
+                    max_size: _Optional[int] = None,
+                    context: _Optional[_Context] = None
+                    ) -> _Strategy[_Contour]:
     """
     Returns a strategy for convex contours.
     Convex contour is a contour such that the line segment
@@ -443,6 +461,7 @@ def convex_contours(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for contour size.
     :param max_size: upper bound for contour size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -505,7 +524,8 @@ def convex_contours(x_coordinates: _Strategy[_Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, _MinContourSize.CONVEX)
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_convex_vertices_sequences(x_coordinates, y_coordinates,
                                        min_size=max(min_size,
                                                     _MinContourSize.CONVEX),
@@ -518,7 +538,9 @@ def concave_contours(x_coordinates: _Strategy[_Coordinate],
                      y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
                      *,
                      min_size: int = _MinContourSize.CONCAVE,
-                     max_size: _Optional[int] = None) -> _Strategy[_Contour]:
+                     max_size: _Optional[int] = None,
+                     context: _Optional[_Context] = None
+                     ) -> _Strategy[_Contour]:
     """
     Returns a strategy for concave contours.
     Concave contour is a contour that is not convex.
@@ -529,6 +551,7 @@ def concave_contours(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for contour size.
     :param max_size: upper bound for contour size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -591,7 +614,8 @@ def concave_contours(x_coordinates: _Strategy[_Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, _MinContourSize.CONCAVE)
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_concave_vertices_sequences(x_coordinates, y_coordinates,
                                         min_size=max(min_size,
                                                      _MinContourSize.CONCAVE),
@@ -601,7 +625,9 @@ def concave_contours(x_coordinates: _Strategy[_Coordinate],
 
 
 def triangular_contours(x_coordinates: _Strategy[_Coordinate],
-                        y_coordinates: _Optional[_Strategy[_Coordinate]] = None
+                        y_coordinates: _Optional[_Strategy[_Coordinate]]
+                        = None,
+                        context: _Optional[_Context] = None
                         ) -> _Strategy[_Contour]:
     """
     Returns a strategy for triangular contours.
@@ -611,6 +637,7 @@ def triangular_contours(x_coordinates: _Strategy[_Coordinate],
     :param y_coordinates:
         strategy for vertices' y-coordinates,
         ``None`` for reusing x-coordinates strategy.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -666,7 +693,8 @@ def triangular_contours(x_coordinates: _Strategy[_Coordinate],
     ...     for vertex in contour.vertices)
     True
     """
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_triangular_vertices_sequences(x_coordinates, y_coordinates,
                                            context=context)
             .map(context.contour_cls))
@@ -674,7 +702,9 @@ def triangular_contours(x_coordinates: _Strategy[_Coordinate],
 
 def rectangular_contours(x_coordinates: _Strategy[_Coordinate],
                          y_coordinates: _Optional[_Strategy[_Coordinate]]
-                         = None) -> _Strategy[_Contour]:
+                         = None,
+                         context: _Optional[_Context] = None
+                         ) -> _Strategy[_Contour]:
     """
     Returns a strategy for axis-aligned rectangular contours.
     Rectangular contour is a contour formed by 4 points.
@@ -683,6 +713,7 @@ def rectangular_contours(x_coordinates: _Strategy[_Coordinate],
     :param y_coordinates:
         strategy for vertices' y-coordinates,
         ``None`` for reusing x-coordinates strategy.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -738,15 +769,16 @@ def rectangular_contours(x_coordinates: _Strategy[_Coordinate],
     ...     for vertex in contour.vertices)
     True
     """
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_rectangular_vertices_sequences(x_coordinates, y_coordinates,
                                             context=context)
             .map(context.contour_cls))
 
 
 def boxes(x_coordinates: _Strategy[_Coordinate],
-          y_coordinates: _Optional[_Strategy[_Coordinate]] = None
-          ) -> _Strategy[_Box]:
+          y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
+          context: _Optional[_Context] = None) -> _Strategy[_Box]:
     """
     Returns a strategy for boxes.
 
@@ -754,6 +786,7 @@ def boxes(x_coordinates: _Strategy[_Coordinate],
     :param y_coordinates:
         strategy for vertices' y-coordinates,
         ``None`` for reusing x-coordinates strategy.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -811,14 +844,15 @@ def boxes(x_coordinates: _Strategy[_Coordinate],
     True
     """
     return _boxes(x_coordinates, y_coordinates,
-                  context=_get_context())
+                  context=_get_context() if context is None else context)
 
 
 def star_contours(x_coordinates: _Strategy[_Coordinate],
                   y_coordinates: _Optional[_Strategy[_Coordinate]] = None,
                   *,
                   min_size: int = _MIN_CONTOUR_SIZE,
-                  max_size: _Optional[int] = None) -> _Strategy[_Contour]:
+                  max_size: _Optional[int] = None,
+                  context: _Optional[_Context] = None) -> _Strategy[_Contour]:
     """
     Returns a strategy for star contours.
     Star contour is a contour such that every vertex is visible from centroid,
@@ -830,6 +864,7 @@ def star_contours(x_coordinates: _Strategy[_Coordinate],
         ``None`` for reusing x-coordinates strategy.
     :param min_size: lower bound for contour size.
     :param max_size: upper bound for contour size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -892,7 +927,8 @@ def star_contours(x_coordinates: _Strategy[_Coordinate],
     True
     """
     _validate_sizes(min_size, max_size, _MIN_CONTOUR_SIZE)
-    context = _get_context()
+    if context is None:
+        context = _get_context()
     return (_star_vertices_sequences(x_coordinates, y_coordinates,
                                      min_size=max(min_size, _MIN_CONTOUR_SIZE),
                                      max_size=max_size,
@@ -906,7 +942,8 @@ def multicontours(x_coordinates: _Strategy[_Coordinate],
                   min_size: int = 0,
                   max_size: _Optional[int] = None,
                   min_contour_size: int = _MIN_CONTOUR_SIZE,
-                  max_contour_size: _Optional[int] = None
+                  max_contour_size: _Optional[int] = None,
+                  context: _Optional[_Context] = None
                   ) -> _Strategy[_Multicontour]:
     """
     Returns a strategy for multicontours.
@@ -922,6 +959,7 @@ def multicontours(x_coordinates: _Strategy[_Coordinate],
     :param min_contour_size: lower bound for contour size.
     :param max_contour_size:
         upper bound for contour size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -1012,7 +1050,9 @@ def multicontours(x_coordinates: _Strategy[_Coordinate],
                           min_contour_size=max(min_contour_size,
                                                _MIN_CONTOUR_SIZE),
                           max_contour_size=max_contour_size,
-                          context=_get_context())
+                          context=(_get_context() 
+                                   if context is None 
+                                   else context))
 
 
 def polygons(x_coordinates: _Strategy[_Coordinate],
@@ -1023,7 +1063,8 @@ def polygons(x_coordinates: _Strategy[_Coordinate],
              min_holes_size: int = 0,
              max_holes_size: _Optional[int] = None,
              min_hole_size: int = _MIN_CONTOUR_SIZE,
-             max_hole_size: _Optional[int] = None) -> _Strategy[_Polygon]:
+             max_hole_size: _Optional[int] = None,
+             context: _Optional[_Context] = None) -> _Strategy[_Polygon]:
     """
     Returns a strategy for polygons.
 
@@ -1037,6 +1078,7 @@ def polygons(x_coordinates: _Strategy[_Coordinate],
     :param max_holes_size: upper bound for holes count, ``None`` for countless.
     :param min_hole_size: lower bound for hole size.
     :param max_hole_size: upper bound for hole size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -1152,7 +1194,7 @@ def polygons(x_coordinates: _Strategy[_Coordinate],
                      max_holes_size=max_holes_size,
                      min_hole_size=max(min_hole_size, _MIN_CONTOUR_SIZE),
                      max_hole_size=max_hole_size,
-                     context=_get_context())
+                     context=_get_context() if context is None else context)
 
 
 def multipolygons(x_coordinates: _Strategy[_Coordinate],
@@ -1165,7 +1207,8 @@ def multipolygons(x_coordinates: _Strategy[_Coordinate],
                   min_holes_size: int = 0,
                   max_holes_size: _Optional[int] = None,
                   min_hole_size: int = _MIN_CONTOUR_SIZE,
-                  max_hole_size: _Optional[int] = None
+                  max_hole_size: _Optional[int] = None, 
+                  context: _Optional[_Context] = None
                   ) -> _Strategy[_Multipolygon]:
     """
     Returns a strategy for multipolygons.
@@ -1185,6 +1228,7 @@ def multipolygons(x_coordinates: _Strategy[_Coordinate],
     :param min_hole_size: lower bound for hole size.
     :param max_hole_size:
         upper bound for polygons' hole size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -1303,7 +1347,6 @@ def multipolygons(x_coordinates: _Strategy[_Coordinate],
                     'max_holes_size')
     _validate_sizes(min_hole_size, max_hole_size, _MIN_CONTOUR_SIZE,
                     'min_hole_size', 'max_hole_size')
-    context = _get_context()
     return _multipolygons(x_coordinates, y_coordinates,
                           min_size=max(min_size, 0),
                           max_size=max_size,
@@ -1314,7 +1357,9 @@ def multipolygons(x_coordinates: _Strategy[_Coordinate],
                           max_holes_size=max_holes_size,
                           min_hole_size=max(min_hole_size, _MIN_CONTOUR_SIZE),
                           max_hole_size=max_hole_size,
-                          context=context)
+                          context=(_get_context()
+                                   if context is None
+                                   else context))
 
 
 def mixes(x_coordinates: _Strategy[_Coordinate],
@@ -1331,8 +1376,8 @@ def mixes(x_coordinates: _Strategy[_Coordinate],
           min_multipolygon_holes_size: int = 0,
           max_multipolygon_holes_size: _Optional[int] = None,
           min_multipolygon_hole_size: int = _MIN_CONTOUR_SIZE,
-          max_multipolygon_hole_size: _Optional[int] = None
-          ) -> _Strategy[_Mix]:
+          max_multipolygon_hole_size: _Optional[int] = None, 
+          context: _Optional[_Context] = None) -> _Strategy[_Mix]:
     """
     Returns a strategy for mixes.
     Mix is a triplet of disjoint multipoint, multisegment and multipolygon.
@@ -1359,6 +1404,7 @@ def mixes(x_coordinates: _Strategy[_Coordinate],
     :param min_multipolygon_hole_size: lower bound for hole size.
     :param max_multipolygon_hole_size:
         upper bound for polygons' hole size, ``None`` for unbound.
+    :param context: strategy context.
 
     >>> from ground.base import get_context
     >>> from hypothesis import strategies
@@ -1583,7 +1629,6 @@ def mixes(x_coordinates: _Strategy[_Coordinate],
     _validate_sizes(min_multipolygon_hole_size, max_multipolygon_hole_size,
                     _MIN_CONTOUR_SIZE, 'min_multipolygon_hole_size',
                     'max_multipolygon_hole_size')
-    context = _get_context()
     return _mixes(
             x_coordinates, y_coordinates,
             min_multipoint_size=max(min_multipoint_size, 0),
@@ -1600,7 +1645,7 @@ def mixes(x_coordinates: _Strategy[_Coordinate],
             min_multipolygon_hole_size=max(min_multipolygon_hole_size,
                                            _MIN_CONTOUR_SIZE),
             max_multipolygon_hole_size=max_multipolygon_hole_size,
-            context=context)
+            context=_get_context() if context is None else context)
 
 
 def _validate_sizes(min_size: int, max_size: _Optional[int],
